
#include "comb/mixedradix-gray.h"

#include "comb/mixedradix-gray2.h"

#include "jjassert.h"

#include "fxttypes.h"
#include "fxtio.h"
#include "fxtiomanip.h"


//% Mixed radix Gray code.

//#define TIMING // uncomment to disable printing

void
print(const char *bla, const ulong *f, ulong n)
{
    if ( bla )  cout << bla;
    cout << "[";
    for (ulong k=0; k<n; ++k)  cout << (long)f[k] << (k<n-1?", ":"");
    cout << "]";
}
// -------------------------


void
do_demo(ulong *m, ulong n)
{
    cout << " n = " << n << endl;
    print(" m = ", m, n);
    cout << endl;
    cout << endl;

    ulong ct = 0;
    mixedradix_gray mg(m, n);
//    mixedradix_gray2 mg(m, n);
    do
    {
        cout << setw(3) << ct << ":  ";
        print(" ", mg.data(), n);
        cout << "   j = " << mg.pos();
        cout << endl;
        ++ct;
    }
    while ( mg.next() );

    cout << endl;
    cout << endl;
}
// -------------------------


#define  ARSZ(x)  (sizeof(x)/sizeof(typeof(x[0])))
#define  DEMO(x)  do_demo(x, ARSZ(x))
int
main()
{
#ifdef TIMING

#define N 11
    ulong m[N];
    for (ulong k=0; k<N; ++k)  m[k] = N+1-k;

//#define N 31
//    ulong m[N];
//    for (ulong k=0; k<N; ++k)  m[k] = 2;

//    DEMO( m );
    ulong ct = 0;
    mixedradix_gray mg(m, N);
//    mixedradix_gray2 mg(m, N);
    do  { ++ct; }  while ( mg.next() );
    cout << " ct=" << ct << endl;

#else
//    ulong m1[] = {2, 2, 2, 2};  // same as binary gray code
    ulong m2[] = {4, 3, 2};
    ulong m3[] = {3, 3};

    DEMO( m3 );
    DEMO( m2 );
#endif

    return 0;
}
// -------------------------

/*
 Timing:

---- factorial base:
 time ./bin  // class mixedradix_gray (N==11)
 ct=479001600 == 12!
./bin  2.63s user 0.02s system 99% cpu 2.656 total
12!/2.656 == 180,346,987 numbers per second

 time ./bin  // class mixedradix_gray2 (N==11)
 ct=479001600
./bin  2.20s user 0.02s system 100% cpu 2.217 total
12!/2.217 == 216,058,457 numbers per second


---- binary:
 time ./bin  // class mixedradix_gray (N==31)
 ct=2147483648
./bin  13.94s user 0.06s system 99% cpu 14.005 total
2147483648/14.005 == 153,336,925 numbers per second

 time ./bin  // class mixedradix_gray2 (N==31)
 ct=2147483648
./bin  17.01s user 0.08s system 99% cpu 17.098 total
2147483648/17.098 == 125,598,528 numbers per second

*/
